@@ -34,7 +34,9 @@
 #include <glib/gstdio.h>
 #include <glib/gi18n.h>
 #include <gio/gio.h>
-#include <libhal.h>
+#if defined(HAVE_HAL)
+  #include <libhal.h>
+#endif	
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-bindings.h>
 #include <dbus/dbus-glib-lowlevel.h>
@@ -330,9 +332,11 @@ _get_usb_intfnum_and_properties (DBusGProxy *obex_manager, const char *device, c
   int usb_bus_num;
   int usb_device_num;
   int usb_intf_num;
+#ifdef HAVE_HAL
   DBusError dbus_error;
   DBusConnection *dbus_connection;
   LibHalContext *hal_ctx;
+#endif
   int ods_intf_num = 1;
   int n;
 
@@ -347,6 +351,7 @@ _get_usb_intfnum_and_properties (DBusGProxy *obex_manager, const char *device, c
 
   g_message ("Parsed '%s' into bus=%d device=%d interface=%d", device, usb_bus_num, usb_device_num, usb_intf_num);
 
+#ifdef HAVE_HAL
   dbus_error_init (&dbus_error);
   dbus_connection = dbus_bus_get_private (DBUS_BUS_SYSTEM, &dbus_error);
   if (dbus_error_is_set (&dbus_error))
@@ -467,6 +472,7 @@ end:
   dbus_connection_close (dbus_connection);
   dbus_connection_unref (dbus_connection);
   dbus_error_free (&dbus_error);
+#endif /* HAVE_HAL */
 
   return ods_intf_num;
 }
@@ -496,9 +502,28 @@ g_vfs_backend_obexftp_finalize (GObject *object)
 static void
 g_vfs_backend_obexftp_init (GVfsBackendObexftp *backend)
 {
+  DBusConnection *conn;
+  DBusGConnection *connection;
+  DBusError error;
   GError *err = NULL;
 
-  backend->connection = dbus_g_bus_get (DBUS_BUS_SESSION, &err);
+  /* Otherwise dbus-glib doesn't setup it value types */
+  connection = dbus_g_bus_get (DBUS_BUS_SESSION, NULL);
+
+  if (connection != NULL)
+        dbus_g_connection_unref (connection);
+
+  /* Connect to the session bus via a private connection */
+  dbus_error_init (&error);
+  conn = dbus_bus_get_private (DBUS_BUS_SESSION, &error);
+  if (conn == NULL) {
+      g_printerr ("Connecting to session bus failed: %s\n", err->message);
+      dbus_error_free (&error);
+      return;
+  }
+  dbus_connection_setup_with_g_main (conn, NULL);
+
+  backend->connection = dbus_connection_get_g_connection (conn);
   if (backend->connection == NULL) {
       g_printerr ("Connecting to session bus failed: %s\n", err->message);
       g_error_free (err);
@@ -1528,8 +1553,9 @@ push_transfer_started_cb (DBusGProxy *proxy,
 
   op_backend->status = ASYNC_RUNNING;
   job_data->total_bytes = (goffset) total_bytes;
-  job_data->progress_callback (0, job_data->total_bytes,
-                               job_data->progress_callback_data);
+  if (job_data->progress_callback)
+    job_data->progress_callback (0, job_data->total_bytes,
+                                 job_data->progress_callback_data);
 
   g_cond_signal (op_backend->cond);
   g_mutex_unlock (op_backend->mutex);
@@ -1561,9 +1587,10 @@ push_transfer_progress_cb (DBusGProxy *proxy,
 
   g_message ("transfer progress");
 
-  job_data->progress_callback ((goffset) bytes_transferred,
-                               job_data->total_bytes,
-                               job_data->progress_callback_data);
+  if (job_data->progress_callback)
+    job_data->progress_callback ((goffset) bytes_transferred,
+                                 job_data->total_bytes,
+                                 job_data->progress_callback_data);
 }
 
 static void
@@ -1808,6 +1835,7 @@ do_push (GVfsBackend *backend,
       push_data_free (job_data);
 
       g_vfs_job_failed_from_error (G_VFS_JOB (job), error);
+      return;
     }
 
   push_data_free (job_data);
@@ -1817,7 +1845,18 @@ do_push (GVfsBackend *backend,
    */
   _invalidate_cache_helper (op_backend);
 
-  g_vfs_job_succeeded (G_VFS_JOB (job));
+  if (remove_source && g_unlink (local_path) == -1)
+    {
+      int errsv = errno;
+
+      g_vfs_job_failed (G_VFS_JOB (job),
+                        G_IO_ERROR,
+                        g_io_error_from_errno (errsv),
+                        _("Error deleting file: %s"),
+                        g_strerror (errsv));
+    }
+  else
+    g_vfs_job_succeeded (G_VFS_JOB (job));
 
   op_backend->doing_io = FALSE;
   g_mutex_unlock (op_backend->mutex);
